import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Shapes
import Qt.labs.qmlmodels

import "../components"
import "./view"

import Artix.Handlers
import Artix.Core
import Artix.Gui
import Artix.Models

/**
 * @brief Диалог для многострочного ввода текста
 */
BasicView {
    id: root

    padding: 0
    leftPadding: 24
    rightPadding: 24
    implicitWidth: screenWidth
    implicitHeight: screenHeight
    readonly property bool fullScreen: true
    property var fieldsById: ({})
    property var rowComponents: ({})
    property real delegateHeight: 64
    property real delegateSpacing: 12
    property BasicTextField eventReceiver: null
    property DialogEventFilter eventFilter: DialogEventFilter {
        onKeyboardData: (data, rawData, source) => {
                            let reciever = eventReceiver ? eventReceiver : fieldsById[params.fields[0].fieldId]
                            if (!handler.testAllowDevices(source))
                                reciever.text = ""

                            if (!params.fieldDistributionRegexp
                                    || !handler.distributeData(data, params.fieldDistributionRegexp, fieldsById))
                                reciever.text = data

                            if (params.returnOnValidScan)
                                onOk();
                        }
        onOk: accept()
        onCancel: reject()
    }
    // Базовый делегат для поля ввода
    component InputDelegate: BasicTextField {
        id: inputDelegate
        objectName: field.fieldId
        readOnly: !field.editable
        Style.class: validValue ? "inputMultiText" : "inputMultiTextInvalid"
        padding: 12

        function addComponentToRow() {
            if (!rowComponents[fieldRow])
                rowComponents[fieldRow] = {}
            if (!rowComponents[fieldRow][fieldColumn])
                rowComponents[fieldRow][fieldColumn] = this
        }

        x: {
            addComponentToRow()
            let res = 0
            for (let i = 0; i < fieldColumn; ++i)
                res += rowComponents[fieldRow][i].width + delegateSpacing
            return res
        }
        y: fieldRow * (delegateHeight + delegateSpacing)

        height: delegateHeight
        width: {
            addComponentToRow()
            if (fixedWidth)
                return maskWidth + leftPadding + rightPadding + imageSize
            /**
             * Поля без маски занимают все свободное пространство.
             * Если таких полей больше 1, то они делят пространство поровну
             */
            let w = parent.width
            let rowLen = Object.values(rowComponents[fieldRow]).length
            if (rowLen < 2)
                return w
            let fillCount = 0
            for (let i = 0; i < rowLen; ++i) {
                if (rowComponents[fieldRow][i].fixedWidth)
                    w = w - rowComponents[fieldRow][i].width - delegateSpacing
                else
                    fillCount++
            }
            w = w / fillCount - delegateSpacing * (fillCount - 1) / fillCount
            return w
        }

        property real imageSize: 30
        property string mask: ""
        property string defaultText: field.defaultValue
        property bool validValue: acceptableInput && (field.required && text.length != 0) || !field.required || noTries
        property bool noTries: true
        onTextChanged: noTries = false
        property int fieldType: type

        // Для полей с маской, ширина поля определяется длиной текста маски
        TextMetrics {
            id: metric
            text: inputDelegate.mask
            font: inputDelegate.font
        }
        property alias maskWidth: metric.width
        property bool fixedWidth: maskWidth

        // Определить позицию символа заполнителя
        function blankCharPos() {
            if (!inputMask)
                return -1
            let blankChar = inputMask[inputMask.length - 1]
            if (blankChar === ';')
                blankChar = ' '
            let cursorPos = -1
            for (let i = 0; i < displayText.length; ++i) {
                if (displayText[i] === blankChar) {
                    cursorPos = i
                    break
                }
            }
            return cursorPos
        }
        onFocusChanged: {
            if (focus) {
                eventReceiver = this
                if (!text)
                    text = defaultText
                if (mask && !inputMask)
                    inputMask = mask
            } else {
                eventReceiver = null
            }
        }

        Image {
            anchors{ right: parent.right; rightMargin: 12; verticalCenter: parent.verticalCenter }
            width: imageSize; height: imageSize
            source: Style.parameter("icon.textFieldClear")
            visible: inputDelegate.text
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    inputDelegate.focus = false
                    inputDelegate.inputMask = ""
                    inputDelegate.clear()
                }
            }
        }
        Tr on placeholderText { tr: field.description }
        onCursorPositionChanged: {
            let pos = blankCharPos()
            if (pos >= 0 && pos !== cursorPosition)
                cursorPosition = pos
        }
        Component.onCompleted: {
            if (!field.editable)
                return
            if (!root.fieldsById)
                root.fieldsById = {}
            root.fieldsById[objectName] = this
        }
    }
    // Делегат для ввода произвольного текста
    component StringDelegate: InputDelegate {
        validator: RegularExpressionValidator { regularExpression: Utils.anchoredPattern(field.regexp) }
        mask: field.inputMask
    }
    // Делегат для ввода даты
    component DateDelegate: InputDelegate {
        mask: "99.99.9999;_"
        property string dateFormat: "dd.MM.yyyy"
        validator: DateTimeValidator {
            minimumDate: field.minDate
            maximumDate: field.maxDate
            dateTimeFormat: dateFormat
        }
    }
    // Делегат для ввода даты и времени
    component DateTimeDelegate: InputDelegate {
        mask: Utils.dateTimeFormatToMask(field.dateFormat, '_')
        defaultText: (new Date()).toLocaleString(Locale.LongFormat, field.dateFormat)
        validator: DateTimeValidator { dateTimeFormat: field.dateFormat }
    }
    // Делегат выбора с помощью RadioButton
    component ChoiceDelegate: Item {
        objectName: field.fieldId
        height: delegateHeight
        width: buttons.implicitWidth

        x: {
            if (!rowComponents[fieldRow])
                rowComponents[fieldRow] = {}
            rowComponents[fieldRow][fieldColumn] = this
            let res = 0
            for (let i = 0; i < fieldColumn; ++i)
                res += rowComponents[fieldRow][i].width + delegateSpacing
            return res
        }
        y: fieldRow * (delegateHeight + delegateSpacing)

        property bool noTries: false
        property bool validValue: true
        property int fieldType: type
        property int choiceValue: group.checkedButton ? group.checkedButton.buttonIndex : -1
        property bool fixedWidth: true
        ButtonGroup { id: group; buttons: buttons.children }
        RowLayout {
            id: buttons
            spacing: 9
            Repeater {
                id: repeater
                model: field.labels.count
                delegate: BasicRadioButton {
                    Style.class: "inputMultiText"
                    checked: index === 0
                    property int buttonIndex: index
                    Tr { tr: field.labels.value(index) }
                    Line {
                        visible: index !== 0
                        anchors { right: parent.left; rightMargin: 7 }
                        height: parent.height
                        strokeStyle: ShapePath.DashLine
                        direction: LineDirection.Vertical
                        dashPattern: [2, 3]
                        strokeWidth: 1
                        color: Style.parameter("color.gray")
                    }
                }
            }
        }
        Component.onCompleted: root.fieldsById[objectName] = this
    }

    contentItem: ColumnLayout {
        spacing: 0
        RowLayout {
            BasicLabel {
                Style.class: "inputMultiTextTitle"
                Layout.preferredWidth: contentItem.width / 2
                Layout.topMargin: 16
                Tr { tr: params.title }
            }
            TopPanel {
                Layout.preferredWidth: contentItem.width / 2
                hideUserName: true
                sideMenuButton.anchors.rightMargin: -9
            }
        }
        Item { Layout.preferredHeight: delegateSpacing * 2 }
        Item {
            Layout.fillWidth: true
            Layout.fillHeight: true
            Repeater {
                model: InputMultiTextModel { fields: params.fields }
                delegate: DelegateChooser {
                    role: "type"
                    DelegateChoice {
                        roleValue: InputMultiTextModel.String
                        delegate: StringDelegate {}
                    }
                    DelegateChoice {
                        roleValue: InputMultiTextModel.Date
                        delegate: DateDelegate {}
                    }
                    DelegateChoice {
                        roleValue: InputMultiTextModel.DateTime
                        delegate: DateTimeDelegate {}
                    }
                    DelegateChoice {
                        roleValue: InputMultiTextModel.Choice
                        delegate: ChoiceDelegate {}
                    }
                }
            }
        }
    }

    footer: ColumnLayout {
        spacing: 0
        Rectangle {
            Layout.fillWidth: true; height: 116
            color: Style.parameter("color.modalBackground")
            RowLayout {
                anchors {
                    right: parent.right
                    verticalCenter: parent.verticalCenter
                    rightMargin: 24
                }
                spacing: 8
                BasicButton {
                    Layout.preferredHeight: 68
                    Layout.preferredWidth: 120
                    textLayout {
                        fillWidth: true
                        fillHeight: true
                    }
                    horizontalAlignment: Text.AlignLeft
                    verticalAlignment: Text.AlignBottom
                    visible: !params.hideCancelButton
                    Style.class: "inputMultiTextCancel"
                    Tr { code: "cancelButton"; defaultMessage: "Отмена" }
                    onClicked: reject()
                }
                BasicButton {
                    Layout.preferredHeight: 68
                    Layout.preferredWidth: 168
                    textLayout {
                        fillWidth: true
                        fillHeight: true
                    }
                    horizontalAlignment: Text.AlignLeft
                    verticalAlignment: Text.AlignBottom
                    Style.class: "inputMultiTextOk"
                    Tr { code: "saveButton"; defaultMessage: "Сохранить" }
                    onClicked: accept()
                }
            }
            // Тень
            Rectangle {
                anchors.bottom: parent.top
                width: parent.width
                height: 5
                gradient: Gradient {
                    orientation: Gradient.Vertical
                    GradientStop { position: 0.0; color: "transparent" }
                    GradientStop { position: 1.0; color: Style.parameter("color.shadow") }
                }
                opacity: 0.3
            }
        }
        BottomPanel { id: bottomPanel; Layout.fillWidth: true }
    }

    // !!!!!!!!!! Клавиатура !!!!!!!!!!
    PopupKeyboard {
        id: popupKeyboard
        hidedY: root.height
        shownY: root.height - popupKeyboard.height - bottomPanel.height
        keyboard {
            showCancel: !params.hideCancelButton
            layoutType: params.useDigitalKeyboard ? "input_digits" : "main"
            enterTr: Utils.makeTr("saveButton", "Сохранить")
            onHide: {
                eventReceiver.focus = false
                eventReceiver = null
            }
            onEnter: accept()
            onCancel: reject()
        }
        Connections {
            target: root
            function onEventReceiverChanged() {
                if (eventReceiver != null)
                    popupKeyboard.open()
                else
                    popupKeyboard.close()
            }
        }
    }

    DialogHandler {
        id: handler
        allowDevices: params.allowDevices
    }

    function accept() {
        let fields = Object.values(fieldsById)
        let valid = true
        let result = {}
        for (let i = 0; i < fields.length; ++i) {
            fields[i].noTries = false
            valid = valid && fields[i].validValue
            if (valid) {
                switch (fields[i].fieldType) {
                case InputMultiTextModel.String:
                    result[fields[i].objectName] = fields[i].text
                    break
                case InputMultiTextModel.Date:
                    result[fields[i].objectName] = Date.fromLocaleString(Qt.locale(), fields[i].text, fields[i].dateFormat)
                    break
                case InputMultiTextModel.DateTime:
                    result[fields[i].objectName] = fields[i].text
                    break
                case InputMultiTextModel.Choice:
                    result[fields[i].objectName] = fields[i].choiceValue
                }
            }
        }
        if (!valid)
            return

        handler.sendAnswer({"data": result})
    }

    function reject() {
        handler.sendAnswer({})
    }
}
